/*
 * reader_util.c
 * author: Hector Corrada Bravo (hcorrada@gmail.com)
 * created: Aug 11, 2009
 *
 * Utilities used by spch file reader
 *
 */
#include "spchreader.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/*
 * void print_fileInfo(const fileInfo *file_info)
 *
 * A silly printer of spch file information
 *
 * args:
 *    file_info: structure containing spch file info
 *
 * returns void
 */
void print_fileInfo(fileInfo *file_info)
{
  int i, j;
  panelInfo *panel_info;

  printf("Info on file %s\n", file_info->filename);
  printf("Number of panels: %d\n", file_info->npanels);

  for (i=0; i<file_info->npanels; i++) {
    panel_info = file_info->panel_info+i;

    printf("Panel %s\n", panel_info->name);
    // GET BACK TO THIS
    printf("primer cycles: %d, nbeads %d\n", panel_info->nprimer_cycles, panel_info->nbeads);
    printf("ligation cycles:");
    for (j=0; j<panel_info->nprimer_cycles; j++) {
      printf(" %d", panel_info->nligation_cycles[j]);
    }
    printf("\n");
  }
}

/*
 * void print_fileData(const fileData *file_data)
 *
 * A silly printer of spch file data
 *
 * args:
 *    file_data: structure containing spch file data
 *
 * returns void
 */
void print_fileData(const fileData *file_data)
{
  int i, j, k, l;
  int nbeads;

  panelData *panel_data;
  int offset;
  float **intensities;
  float *cycle_ints;

  char **color_calls;
  char *cycle_calls;

  printf("Data on file %s\n", file_data->filename);
  printf("Number of panels: %d\n", file_data->npanels);

  for (i=0; i<file_data->npanels; i++) {
    panel_data = file_data->panel_data+i;

    printf("Panel %s\n", panel_data->name);
    printf("primer cycles: %d, beads %d\n", panel_data->nprimer_cycles, panel_data->nbeads);
    printf("nligation cycles:");
    for (j=0; j<panel_data->nprimer_cycles; j++) {
      printf(" %d", panel_data->nligation_cycles[j]);
    }
    printf("\n");

    printf("primer names:");
    for (j=0; j<panel_data->nprimer_cycles; j++) {
      printf(" %s", panel_data->primer_names[j]);
    }
    printf("\n");

    printf("ligation names:");
    for (j=0; j<panel_data->total_nligation_cycles; j++) {
      printf(" %s", panel_data->ligation_names[j]);
    }
    printf("\n");

    printf("head of yx location:\n"); 
    for (j=0; j<10; j++) { 
      printf("%d: %d %d\n", j, panel_data->yxloc[j], panel_data->yxloc[panel_data->nbeads+j]); 
    }

    intensities = panel_data->intensities;
    color_calls = panel_data->color_calls;
    nbeads = panel_data->nbeads;

    offset = 0;
    for (j=0; j<3; j++) {
      for (k=0; k<5; k++) {
	printf("primer %s, ligation %s:\n", panel_data->primer_names[j], panel_data->ligation_names[offset + k]);
	
	cycle_ints = intensities[offset + k];
	cycle_calls = color_calls[offset + k];
	for (l=0; l<10; l++) {
	  printf("%d: %f %f %f %f: ", l, cycle_ints[l], cycle_ints[l+nbeads], cycle_ints[l+2*nbeads], cycle_ints[l+3*nbeads]);
	  printf("%hd\n", cycle_calls[l]);
	}

	printf("\n");
      }
      offset += panel_data->nligation_cycles[j];
    }
  }
}


/*
 * frees space allocated to a  panelInfo structure
 */
static void free_panelInfo(panelInfo *panel_info)
{
  free(panel_info->nligation_cycles);
}

/*
 * frees space allocated to a fileInfo structure
 */
void free_fileInfo(fileInfo *file_info)
{
  int i;

  for (i=0; i<file_info->npanels; i++) {
    free_panelInfo(file_info->panel_info+i);
  }
  free(file_info->panel_info);
}

