/*
 * spchreader.h
 * author: Hector Corrada Bravo (hcorrada@gmail.com)
 * created: Aug 11, 2009
 *
 * type definitions and function declarations for spch file reader
 *
 */
#ifndef __SPCHREADER_H
#define __SPCHREADER_H

#define NAMESIZE 1024
#define NPRIMER_CYCLES 5

/* include hdf5 library headers */
#include "hdf5.h"
#include "H5Gpublic.h"

/* Information on a panel of data */
typedef struct _panelInfo {
  char name[NAMESIZE]; 
  int nprimer_cycles;
  int total_nligation_cycles;
  int *nligation_cycles;

  int nbeads;
} panelInfo;

/* Information on an spch file */
typedef struct _fileInfo {
  char filename[NAMESIZE];
  int npanels;
  int next;
  panelInfo *panel_info;
} fileInfo;

/* Container for one panel's worth of data */
typedef struct _panelData {
  char name[NAMESIZE];

  int next_primer;
  int next_ligation;

  int nprimer_cycles;
  int total_nligation_cycles;
  int *nligation_cycles;
  int nbeads;

  char **primer_names;
  char **ligation_names;
  float **intensities;
  int *yxloc;
  char **color_calls;
} panelData;

/* Container for spch file data */
typedef struct _fileData {
  const char *filename;
  int npanels;
  int next;
  panelData *panel_data;
} fileData;

/* Frees memory used by fileInfo structure */
void free_fileInfo (fileInfo *file_info);

/* A silly printer for file info */
void print_fileInfo (fileInfo *file_info);

/* A silly printer for file data */
void print_fileData (const fileData *file_data);

/* 
 * This function gets information about the spch file and
 * stores it in a structure of type fileInfo
 *
 * This is step 1 in reading the spch file, file_info can be used
 * to allocate space for the file data using whatever memory allocation
 * procedure the user of this library prefers (e.g., R_alloc in R)
 */
herr_t getspchinfo (const char *filename, fileInfo *file_info, int verbose);

/*
 * This function actually reads in the spch file data
 * and places it in the given fileData structure
 */
herr_t readspch (const char *filename, fileData *file_data, int verbose);
#endif
