#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <R.h>
#include <Rmath.h>
#include <R_ext/PrtUtil.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include <R_ext/Rdynload.h>
#include <string.h>

static inline char char_score(const double ratio,const int max, const int min, const int offset);

/* get maximum/minimum values for each row/col */
SEXP rowMax(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(REALSXP, nrow));
  double *dtmp = REAL(res);
  double *input=REAL(mat);

  for(i=0; i<nrow; i++) 
    dtmp[i]=input[i];

  for(j=1; j<ncol; j++) { /* loop for columns */
    tmpidx = j*nrow; 
    for(i=0; i<nrow; i++) { /* loop for rows */
      if(dtmp[i]<input[tmpidx+i]) 
	dtmp[i]=input[tmpidx+i];
    }
  }
  UNPROTECT(1);
  return(res);
}

SEXP rowMin(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(REALSXP, nrow));
  double *dtmp = REAL(res);
  double *input=REAL(mat);

  for(i=0; i<nrow; i++)
    dtmp[i]=input[i];

  for(j=1; j<ncol; j++) { /* loop for columns */
    tmpidx = j*nrow;
    for(i=0; i<nrow; i++) { /* loop for rows */
      if(dtmp[i]>input[tmpidx+i])
        dtmp[i]=input[tmpidx+i];
    }
  }
  UNPROTECT(1);
  return(res);
}

SEXP colMax(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(REALSXP, ncol));
  double *dtmp = REAL(res);
  double *input=REAL(mat);

  for(j=0; j<ncol; j++) { /* loop for columns */
    tmpidx = j*nrow;
    dtmp[j] = input[tmpidx];
    for(i=1; i<nrow; i++) { /* loop for rows */
      if(dtmp[j]<input[tmpidx+i])
        dtmp[j]=input[tmpidx+i];
    }
  }
  UNPROTECT(1);
  return(res);
}

SEXP colMin(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(REALSXP, ncol));
  double *dtmp = REAL(res);
  double *input=REAL(mat);

  for(j=0; j<ncol; j++) { /* loop for columns */
    tmpidx = j*nrow;
    dtmp[j] = input[tmpidx];
    for(i=1; i<nrow; i++) { /* loop for rows */
      if(dtmp[j]>input[tmpidx+i])
        dtmp[j]=input[tmpidx+i];
    }
  }
  UNPROTECT(1);
  return(res);
}

/* Get the position of maximum value for each column.
   This can be used in calling bases/colors.
   Positions are integers from 1. */
SEXP colMaxPos(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(INTSXP, ncol));
  itmp = INTEGER(res);
  double *input=REAL(mat), dtmp;

  for(j=0; j<ncol; j++) { 
    tmpidx = j*nrow;
    itmp[j] = 1;
    dtmp = input[tmpidx];
    for(i=1; i<nrow; i++) {
      if(dtmp<input[tmpidx+i]) {
        itmp[j]=i+1;
	dtmp = input[tmpidx+i];
      }
    }
  }
  UNPROTECT(1);
  return(res);
}

SEXP rowMaxPos(SEXP mat) {
  int i, j, tmpidx;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol));
  int nrow = itmp[0], ncol=itmp[1];
  SEXP res;
  PROTECT(res = allocVector(INTSXP, nrow));
  itmp = INTEGER(res);
  double *input=REAL(mat), *dtmp;
  dtmp=(double *)R_alloc(nrow, sizeof(double));

  for(i=0; i<nrow; i++) {
    dtmp[i]=input[i];
    itmp[i]=1;
  }

  for(j=1; j<ncol; j++) { /* loop for columns */
    tmpidx = j*nrow; 
    for(i=0; i<nrow; i++) { /* loop for rows */
      if(dtmp[i]<input[tmpidx+i]) {
	dtmp[i] = input[tmpidx+i];
        itmp[i]=j+1;
      }
    }
  }
  UNPROTECT(1);
  return(res);
}

/* call bases/colors from 3D matrix (nread x ncolor x ncycle).
   This returns nread strings, each is ncycle long.
   This is works for single or two color systems. */
SEXP callBases(SEXP mat, SEXP type, SEXP score) {
  int iread, icolor, icycle, tmpidx, tmpidx2;
  double *input=REAL(mat), dtmp, dtmp2, dscore;
  int *itmp = INTEGER(getAttrib(mat, R_DimSymbol)), itmp2;

  char call_type = *CHARACTER_VALUE(type);
  char score_type = *CHARACTER_VALUE(score);
  int nread = itmp[0], ncolor=itmp[1], ncycle=itmp[2];
  char *tmpchar, *tmpscore;
  tmpchar = (char*) R_alloc(ncycle+1, sizeof(char));
  tmpchar[ncycle] = '\0';
  
  if(score_type!='n') {
    tmpscore = (char *) R_alloc(ncycle+1, sizeof(int));
    tmpscore[ncycle] = '\0';
  }

  SEXP calls, quals, res;
  PROTECT(calls = allocVector(STRSXP, nread));
  if(score_type!='n') {
    PROTECT(quals = allocVector(STRSXP, nread));
    PROTECT(res = allocVector(VECSXP, 2));
  }

  /* look up table */
  char color_tbl[4]={'0', '1', '2', '3'};
  char nucl_tbl[4]={'A', 'C', 'G', 'T'}; 
  char *tbl;


  /*  Rprintf("%c %c\n", call_type, score_type);*/

  tbl = (call_type == 'n') ? nucl_tbl : color_tbl;
  
  for(iread=0; iread<nread; iread++) { /* call base for this read */
    for(icycle=0; icycle<ncycle; icycle++) {
      tmpidx = icycle*nread*ncolor + iread;
      dtmp = input[tmpidx]; dtmp2=0.0;
      itmp2 = 0;
      /* compare to other colors */
      tmpidx2 = tmpidx;

      /*      Rprintf("iread %d, icycle %d, icolor 0, dtmp %.4f dtmp2 %.4f\n",iread, icycle, dtmp, dtmp2);*/
      for(icolor=1; icolor<ncolor; icolor++) {
	tmpidx2 = tmpidx2+nread;
	if(dtmp < input[tmpidx2]) {
	  dtmp2 = dtmp;
	  dtmp = input[tmpidx2]; 
	  itmp2 = icolor;
	} else {
	  if (dtmp2 < input[tmpidx2])
	    dtmp2 = input[tmpidx2];
	}
	/*	Rprintf("iread %d, icycle %d, icolor %d, p %.4f dtmp %.4f dtmp2 %.4f\n",iread, icycle, icolor, input[tmpidx2], dtmp, dtmp2);*/
      }
      tmpchar[icycle] = tbl[itmp2];

      switch (score_type) {
      case 'p': 
	/* compute phred score */
	tmpscore[icycle] = char_score(1.0/(1.0-dtmp),40,0,33);
	break;
	
      case 'l': 
	/* compute log ratio score */
	tmpscore[icycle] = char_score(dtmp / (1.0-dtmp), 40, -5, 64);
	break;
	
      case 'm':
	/* compute log ratio score, comparing to second max prob */
	tmpscore[icycle] = char_score(dtmp / dtmp2, 40 ,-5, 64);
	/*	Rprintf("iread %d icycle %d dtmp %.4f dtmp2 %.4f dscore %.4f tmpscore %d\n", iread, icycle, dtmp, dtmp2, dscore, tmpscore[icycle]);*/
	break;
      }
    }

    /* assign call  */
    SET_STRING_ELT(calls, iread, mkChar(tmpchar));
    if(score_type!='n') {
      SET_STRING_ELT(quals, iread, mkChar(tmpscore));
    }
  }
  if(score_type!='n') {
    SET_VECTOR_ELT(res, 0, calls);
    SET_VECTOR_ELT(res, 1, quals);
    UNPROTECT(3);
    return(res);
  } 
  else {
    UNPROTECT(1);
    return(calls);
  }
}

/* call bases/colors from 3D matrix (nread x 4 x ncycle)
   but gives alternative calls in positions with high uncertainty
   This returns nread strings, each ncycle long
   a vector of log ratios of length nread*ncycle
   a vector of alternative counts of length nread*ncycle
   nread extra strings of length equal to the sum of the number of alternatives
   a vector of more log ratios of length equal to the sum of the number of alternatives
*/
static inline double entropy(const double p) {
  double res=log2(p);

  res = R_FINITE(res) ? -p * res : 0.0;
  return(res);
}


static inline char char_score(const double ratio, 
			      const int max, const int min, const int offset) {
  double dscore;
  int iscore;

  dscore = 10 * log10(ratio);
  if (R_FINITE(dscore)) {
    iscore = (int) (dscore < 0.0 ? dscore - 0.5 : dscore + 0.5);
  } else {
    iscore = (dscore == R_PosInf) ? max : min;
  }
      
  if (iscore > max) iscore = max;
  if (iscore < min) iscore = min;

  return((char) (iscore+offset));
}
